from .client import JapydClient
from .dotnet import (
    JsonApiBaseModel,
    JsonApiBodyModel,
    JsonApiQueryModel,
    MultiBodyModel,
    SingleBodyModel,
)
from .jsonapi import (
    Error,
    JsonApiApp,
    Link,
    MultiResourcesTopLevel,
    Relationship,
    Resource,
    ResourceIdentifier,
    SingleResourceTopLevel,
    TopLevel,
)

__all__ = [
    "TopLevel",
    "Resource",
    "ResourceIdentifier",
    "Relationship",
    "Link",
    "Error",
    "JsonApiApp",
    "JsonApiBaseModel",
    "SingleBodyModel",
    "MultiBodyModel",
    "JsonApiQueryModel",
    "JsonApiBodyModel",
    "JapydClient",
    "SingleResourceTopLevel",
    "MultiResourcesTopLevel",
]



from enum import StrEnum


class Oper(StrEnum):
    NOT = "not"
    OR = "or"
    AND = "and"

    EQUALS = "equals"
    LESS_THAN = "lessThan"
    LESS_OR_EQUAL = "lessOrEqual"
    GREATER_THAN = "greaterThan"
    GREATER_OR_EQUAL = "greaterOrEqual"

    CONTAINS = "contains"
    STARTS_WITH = "startsWith"
    ENDS_WITH = "endsWith"

    ANY = "any"  # Equals one value from set
    HAS = "has"  # Collection contains items



import typing as t

import requests

from .jsonapi import MultiResourcesTopLevel, SingleResourceTopLevel


class JapydClient:

    def __init__(self, url: str):
        self.url = url.rstrip("/")

    @t.overload
    def __call__(self, method: str, entry: str, /, id: str, **kwargs) -> SingleResourceTopLevel: ...

    @t.overload
    def __call__(self, method: str, entry: str, /, **kwargs) -> MultiResourcesTopLevel: ...

    def __call__(self, method: str, entry: str, /, id=None, **kwargs):
        if id is not None:
            resp = requests.request(method, f"{self.url.rstrip('/')}/{entry.strip('/')}/{id}", **kwargs)
            return SingleResourceTopLevel.model_validate(resp.json())

        resp = requests.request(method, f"{self.url.rstrip('/')}/{entry.strip('/')}", **kwargs)
        return MultiResourcesTopLevel.model_validate(resp.json())
